@@ -3,8 +3,8 @@ import pandas as pd
 from datetime import datetime
 from .factors import KlineGeneratorBy1Min, CzscFactors
 from .data.jq import get_kline, get_kline_period
-from .data import freq_map, freq_inv
-from .enum import Signals, Factors
+from .data import freq_inv
+from .enum import Factors
 
 class CzscTrader:
     """缠中说禅股票 选股/择时"""
@@ -38,24 +38,6 @@ class CzscTrader:
             return s['日线笔因子']
         return "other"
 
-    def run_history(self):
-        """查看第N-3笔的历史形态"""
-        s = self.s
-        nine_values = [x.value for x in Signals.__members__.values() if x.name[:3] in ["X9L", "X9S"]]
-        seven_values = [x.value for x in Signals.__members__.values() if x.name[:3] in ["X7L", "X7S"]]
-        five_values = [x.value for x in Signals.__members__.values() if x.name[:3] in ["X5L", "X5S"]]
-
-        for freq in ["30分钟", "日线"]:
-            if s['{}_倒4的九笔形态'.format(freq)] in nine_values:
-                return "{}_倒4的九笔形态_{}".format(freq, s['{}_倒4的九笔形态'.format(freq)])
-
-            if s['{}_倒4的七笔形态'.format(freq)] in seven_values:
-                return "{}_倒4的七笔形态_{}".format(freq, s['{}_倒4的七笔形态'.format(freq)])
-
-            if s['{}_倒4的五笔形态'.format(freq)] in five_values:
-                return "{}_倒4的五笔形态_{}".format(freq, s['{}_倒4的五笔形态'.format(freq)])
-        return "other"
-
     def take_snapshot(self, file_html, width="1400px", height="680px"):
         self.kf.take_snapshot(file_html, width, height)
 
